/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.mingraviditet.ui.main.overview.compose

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.paint
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import com.trifork.gravidcompose.component.GravidExpandableTopBar
import com.trifork.gravidcompose.component.GravidIcon
import com.trifork.gravidcompose.component.GravidOutlinedButton
import com.trifork.gravidcompose.component.GravidText
import com.trifork.gravidcompose.component.GravidTextButton
import com.trifork.gravidcompose.theme.GravidTheme
import com.trifork.gravidcompose.utils.GravidPreview
import com.trifork.gravidcompose.utils.rememberForeverLazyListState
import com.trifork.gravidcompose.utils.rememberForeverTopAppBarState
import dk.nsp.domain.articles.Article
import dk.nsp.domain.articles.OverviewArticles
import dk.nsp.domain.pregnancymodels.GestationAge
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.main.common.compose.InnerCircularCard
import dk.nsp.mingraviditet.ui.main.overview.OverviewFragmentViewModel
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.FetusProgressWidget
import dk.nsp.mingraviditet.ui.main.overview.compose.widget.OverviewPregnancyInformationCardsRow
import dk.nsp.mingraviditet.ui.main.overview.widget.OverviewPregnancyState
import dk.nsp.shared.util.extensions.toMediumFormat
import java.time.LocalDate

@Composable
fun OverviewRoute(
    overviewViewModel: OverviewFragmentViewModel = hiltViewModel(),
    hasShownProgressAnimation: Boolean = false,
    onNavigateToKnowledge: () -> Unit,
    onNavigateToArticleDetails: (article: Article, sectionId: Int?) -> Unit,
    onNavigateToSetDueDate: () -> Unit,
    onNavigateToSetDateOfBirth: () -> Unit,
) {
    val overviewArticles by overviewViewModel.articles.collectAsState()
    val overviewWidgetState by overviewViewModel.overviewWidgetState.collectAsState()

    OverviewScreen(
        state = overviewWidgetState,
        overviewArticles = overviewArticles,
        onNavigateToKnowledge = onNavigateToKnowledge,
        onNavigateToArticleDetails = onNavigateToArticleDetails,
        hasShownProgressAnimation = hasShownProgressAnimation,
        onNavigateToSetDueDate = onNavigateToSetDueDate,
        onNavigateToSetDateOfBirth = onNavigateToSetDateOfBirth,
        clearCurrentPregnancy = { overviewViewModel.clearPregnancy() },
    )
}

@Composable
fun OverviewScreen(
    state: OverviewPregnancyState,
    overviewArticles: OverviewArticles?,
    onNavigateToKnowledge: () -> Unit,
    onNavigateToArticleDetails: (article: Article, sectionId: Int?) -> Unit,
    onNavigateToSetDueDate: () -> Unit,
    onNavigateToSetDateOfBirth: () -> Unit,
    clearCurrentPregnancy: () -> Unit,
    hasShownProgressAnimation: Boolean = false,
) {
    var internalHasShowProgress by remember {
        mutableStateOf(hasShownProgressAnimation)
    }

    var showClearDialog by remember {
        mutableStateOf(false)
    }

    val lazyState = rememberForeverLazyListState(key = "Overview")
    val barState = rememberForeverTopAppBarState("Overview")
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        state = barState,
        canScroll = { true },
    )
    val scrollConnection = scrollBehavior.nestedScrollConnection

    Scaffold(
        modifier = Modifier
            .fillMaxSize()
            .nestedScroll(scrollConnection)
            .paint(
                painter = painterResource(id = R.drawable.ic_bg_circles_01),
                alignment = Alignment.TopCenter,
                contentScale = ContentScale.FillWidth,
            ),
        containerColor = Color.Transparent,
        topBar = {
            GravidExpandableTopBar(
                title = { Text(text = stringResource(id = R.string.bottom_navigation_title_appointments)) },
                subtitle = { Text(text = stringResource(id = R.string.overview_welcome_message_unauthenticated)) },
                scrollBehavior = scrollBehavior,
                titleHorizontalArrangement = Arrangement.Start,
                expandedBackgroundTransparent = true,
                actions = {
                    if (state is OverviewPregnancyState.GaveBirth) {
                        InnerCircularCard(
                            containerColor = GravidTheme.colorScheme.surface,
                            modifier = Modifier
                                .padding(
                                    end = GravidTheme.dimens.largeSpacing,
                                    top = GravidTheme.dimens.largeSpacing * (1f - scrollBehavior.state.collapsedFraction),
                                ),
                        ) {
                            IconButton(onClick = { showClearDialog = true }) {
                                GravidIcon(
                                    painterResource(id = R.drawable.ic_calendar),
                                    tint = GravidTheme.colorScheme.tertiary,
                                )
                            }
                        }
                    }
                },
            )
        },
    ) { innerPadding ->

        LazyColumn(
            state = lazyState,
            modifier = Modifier
                .padding(innerPadding)
                .fillMaxWidth(),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            item {
                FetusProgressWidget(
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = GravidTheme.dimens.largeSpacing),
                    size = 250.dp,
                    monthsIntoPregnancy = state.pregnancyMonthDouble,
                    animateProgress = !internalHasShowProgress,
                    onProgressAnimationFinished = { internalHasShowProgress = true },
                )
            }

            item {
                when (state) {
                    is OverviewPregnancyState.GaveBirth -> {
                        Row(verticalAlignment = Alignment.CenterVertically) {
                            GravidText(
                                text = stringResource(id = R.string.overview_birth_date, state.dateOfBirth.toMediumFormat()),
                                style = GravidTheme.typography.Header4,
                            )

                            IconButton(
                                onClick = onNavigateToSetDateOfBirth,
                            ) {
                                GravidIcon(
                                    painter = painterResource(id = R.drawable.ic_edit),
                                    tint = GravidTheme.colorScheme.tertiary,
                                )
                            }
                        }
                    }

                    is OverviewPregnancyState.WaitingForBirth -> {
                        Row(verticalAlignment = Alignment.CenterVertically) {
                            GravidText(
                                text = stringResource(id = R.string.overview_your_pregnancy_due_date, state.expectedDueDate.toMediumFormat()),
                                style = GravidTheme.typography.Header4,
                            )

                            IconButton(onClick = onNavigateToSetDueDate) {
                                GravidIcon(
                                    painter = painterResource(id = R.drawable.ic_edit),
                                    tint = GravidTheme.colorScheme.tertiary,
                                )
                            }
                        }

                        val dateOfBirthComposable: (@Composable () -> Unit)? = if (state.week >= 36) {
                            {
                                GravidOutlinedButton(
                                    contentPadding = PaddingValues(
                                        horizontal = GravidTheme.dimens.largeSpacing,
                                        vertical = 13.dp,
                                    ),
                                    onClick = onNavigateToSetDateOfBirth,
                                    text = stringResource(id = R.string.overview_did_give_birth_button_title),
                                )
                            }
                        } else {
                            null
                        }

                        OverviewPregnancyInformationCardsRow(
                            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                            pregnancyWeek = state.week,
                            gestationAge = state.gestationAge,
                            pregnancyMonth = state.pregnancyMonth,
                            trimester = state.trimester,
                            trailingComponent = dateOfBirthComposable,
                        )
                    }

                    is OverviewPregnancyState.None -> {
                        GravidText(
                            modifier = Modifier.padding(top = GravidTheme.dimens.defaultSpacing),
                            text = stringResource(id = R.string.overview_your_pregnancy_title),
                            style = GravidTheme.typography.Header4,
                        )

                        GravidOutlinedButton(
                            modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing),
                            onClick = onNavigateToSetDueDate,
                            text = stringResource(id = R.string.overview_your_pregnancy_select_due_date),
                            trailingIcon = R.drawable.ic_calendar,
                        )
                    }
                }
            }

            if (overviewArticles?.currentWeek?.isNotEmpty() == true) {
                item {
                    ArticleHeader(
                        modifier = Modifier.padding(top = GravidTheme.dimens.largeSpacing, start = GravidTheme.dimens.largeSpacing),
                        title = stringResource(id = R.string.overview_relevant_right_now_title),
                        trailingTitle = stringResource(id = R.string.overview_find_more_knowledge_button),
                        trailingOnClick = onNavigateToKnowledge,
                    )
                }

                items(overviewArticles.currentWeek) {
                    ArticleCard(
                        modifier = Modifier
                            .padding(horizontal = GravidTheme.dimens.largeSpacing)
                            .padding(bottom = GravidTheme.dimens.defaultSpacing),
                        article = it,
                        onClick = {
                            onNavigateToArticleDetails(it.article, it.articleSectionId)
                        },
                    )
                }
            }

            if (overviewArticles?.nextWeek?.isNotEmpty() == true) {
                item {
                    ArticleHeader(
                        modifier = Modifier.padding(start = GravidTheme.dimens.largeSpacing),
                        title = stringResource(id = R.string.overview_relevant_next_week_title),
                    )
                }

                items(overviewArticles.nextWeek ?: emptyList()) {
                    ArticleCard(
                        modifier = Modifier
                            .padding(horizontal = GravidTheme.dimens.largeSpacing)
                            .padding(bottom = GravidTheme.dimens.defaultSpacing),
                        article = it,
                        onClick = {
                            onNavigateToArticleDetails(it.article, it.articleSectionId)
                        },
                    )
                }
            }
        }
    }

    if (showClearDialog) {
        AlertDialog(
            containerColor = GravidTheme.colorScheme.surface,
            onDismissRequest = { showClearDialog = false },
            confirmButton = {
                GravidTextButton(
                    onClick = {
                        clearCurrentPregnancy()
                        showClearDialog = false
                    },
                    textStyle = GravidTheme.typography.Header4,
                    text = stringResource(id = R.string.common_delete),
                )
            },
            title = { GravidText(text = stringResource(id = R.string.overview_start_new_pregnancy_button_title), style = GravidTheme.typography.Header4) },
            text = { GravidText(text = stringResource(id = R.string.overview_start_new_pregnancy_alert_description)) },
            dismissButton = {
                GravidTextButton(
                    text = stringResource(id = R.string.common_cancel),
                    onClick = { showClearDialog = false },
                    textStyle = GravidTheme.typography.Header4,
                )
            },
        )
    }
}

@GravidPreview
@Composable
fun OverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            overviewArticles = OverviewArticles(emptyList(), emptyList()),
            onNavigateToKnowledge = { /*TODO*/ },
            onNavigateToSetDateOfBirth = {},
            onNavigateToArticleDetails = { _, _ -> },
            onNavigateToSetDueDate = {},
            clearCurrentPregnancy = {},
            hasShownProgressAnimation = true,
            state = OverviewPregnancyState.WaitingForBirth(
                expectedDueDate = LocalDate.now().plusMonths(1),
                week = 38,
                pregnancyMonth = 8,
                trimester = 3,
                gestationAge = GestationAge(35, 5),
                monthsIntoPregnancy = 8.4,
            ),
        )
    }
}

@GravidPreview
@Composable
fun NoEDDOverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            onNavigateToKnowledge = { },
            onNavigateToArticleDetails = { _, _ -> },
            onNavigateToSetDueDate = { },
            onNavigateToSetDateOfBirth = {},
            clearCurrentPregnancy = {},
            overviewArticles = OverviewArticles(emptyList(), emptyList()),
            hasShownProgressAnimation = true,
            state = OverviewPregnancyState.None,
        )
    }
}

@GravidPreview
@Composable
fun GaveBirthOverviewScreenPreview() {
    GravidTheme {
        OverviewScreen(
            state = OverviewPregnancyState.GaveBirth(dateOfBirth = LocalDate.now()),
            overviewArticles = OverviewArticles(emptyList(), emptyList()),
            onNavigateToKnowledge = { /*TODO*/ },
            onNavigateToArticleDetails = { _, _ -> },
            onNavigateToSetDueDate = { /*TODO*/ },
            onNavigateToSetDateOfBirth = { /*TODO*/ },
            hasShownProgressAnimation = true,
            clearCurrentPregnancy = {},
        )
    }
}
