/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021-2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import dk.nsp.api.cms.CMSDataSource
import dk.nsp.domain.articles.Article
import dk.nsp.domain.articles.ArticleCategory
import dk.nsp.domain.articles.OverviewArticles
import dk.nsp.domain.articles.toDomainModel
import dk.nsp.mingraviditet.database.dao.ArticleCategoryDao
import dk.nsp.mingraviditet.database.dao.ArticleDao
import dk.nsp.mingraviditet.database.dao.ArticleSectionDao
import dk.nsp.models.toEntity
import dk.nsp.shared.Failed
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.Success
import dk.nsp.shared.asSuccessValueOrNull
import dk.nsp.shared.mapSuccess
import dk.nsp.shared.mapSuccessWithTryOrFailed
import dk.nsp.util.searchResults
import kotlinx.coroutines.flow.map
import javax.inject.Inject

class ArticlesRepository @Inject constructor(
    private val cmsDataSource: CMSDataSource,
    private val articleCategoriesDao: ArticleCategoryDao,
    private val articlesDao: ArticleDao,
    private val articleSectionDao: ArticleSectionDao,
) {

    suspend fun getOverviewArticles(week: Int?): OverviewArticles? {
        return if (week != null) {
            when (val res = cmsDataSource.getArticleCategories(week)) {
                is Failed -> null
                is Success -> {
                    val recommendedArticleCategory = res.value.toDomainModel()

                    OverviewArticles(
                        currentWeek = recommendedArticleCategory.currentWeek.flatMap { it.searchResults() },
                        nextWeek = recommendedArticleCategory.nextWeek?.flatMap { it.searchResults() },
                    )
                }
            }
        } else {
            val randomWeek = (1..10).random()

            when (val articlesRes = cmsDataSource.getArticleCategories(randomWeek)) {
                is Failed -> null
                is Success -> {
                    val recommendedArticleCategory = articlesRes.value.toDomainModel()

                    OverviewArticles(currentWeek = recommendedArticleCategory.currentWeek.flatMap { it.searchResults() })
                }
            }
        }
    }

    suspend fun getAfterBirthArticles(afterBirthWeek: Int): OverviewArticles? {
        return when (val res = cmsDataSource.getAfterBirthArticleCategories(afterBirthWeek.coerceAtMost(8))) {
            is Failed -> null
            is Success -> {
                val recommendedArticleCategory = res.value.toDomainModel()
                OverviewArticles(
                    currentWeek = recommendedArticleCategory.currentWeek.flatMap { it.searchResults() },
                    nextWeek = recommendedArticleCategory.nextWeek?.flatMap { it.searchResults() },
                )
            }
        }
    }

    private var _cache: List<ArticleCategory>? = null

    val categories = articleCategoriesDao.getArticleCategories().map {
        it.map { ArticleCategory(it.entity.name, it.entity.description, it.entity.icon, articles = emptyList(), it.entity.slug) }
    }

    suspend fun getArticleCategories(): ResultWrapper<List<ArticleCategory>> {
        val categoriesRes = cmsDataSource.getArticleSections()

        // Put data into database...
        categoriesRes.asSuccessValueOrNull()?.forEach { category ->
            articleCategoriesDao.insertArticleCategory(category.toEntity())
            category.articles.map { article ->
                articlesDao.insertArticle(article = article.toEntity(category.categoryId))
                article.sections.forEach { section ->
                    articleSectionDao.insertSection(section.toEntity(articleId = article.id))
                }
            }
        }

//        val articlesFromWeekForWeek = articlesDao.getArticlesFrom(13)
//        Timber.d("articlesFromWeekForWeek -> ${articlesFromWeekForWeek.size}")

//        categoriesRes.asSuccessValueOrNull()?.map {
//            it.articles.forEach {
//                it.
//            }
//        }

        return categoriesRes.mapSuccess {
            it.map {
                it.toDomainModel()
            }.apply {
                _cache = this
            }
        }
    }

    suspend fun searchArticles(
        query: String,
    ): ResultWrapper<List<ArticleCategory>> {
        val searchResponse = cmsDataSource.searchArticles(query)

        return searchResponse.mapSuccessWithTryOrFailed { categories ->
            categories.map { it.toDomainModel() }
        }
    }

    suspend fun getInternalArticleByLink(internalLink: String): Pair<Article?, Int> {
        var matchedArticle: Article? = null

        val redirectLocation = cmsDataSource.getRedirectLink(internalLink)

        val articleId = redirectLocation.substringAfterLast("/").toIntOrNull() ?: return Pair(null, -1)

        // If we visit from the Overview screen, nothing is cached.
        if (_cache == null) {
            getArticleCategories()
        }

        // Search for article
        _cache?.forEach OuterForEach@{ articleCategory ->

            articleCategory.articles.forEach { article ->
                if (article.id == articleId) {
                    matchedArticle = article
                    return@OuterForEach
                }

                article.sections.forEach { section ->
                    if (section.id == articleId) {
                        matchedArticle = article
                        return@OuterForEach
                    }
                }
            }
        }

        return Pair(matchedArticle, articleId)
    }
}
