/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2023 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsp.domain.repository

import dk.nsp.api.appointments.AppointmentsOfflineDataSource
import dk.nsp.domain.careplan.SplitCareplanActivities
import dk.nsp.domain.careplan.toDomainModel
import dk.nsp.domain.pregnancymodels.DueDateMethod
import dk.nsp.mingraviditet.datastore.PregnancyPreferencesDataSource
import dk.nsp.shared.Dispatcher
import dk.nsp.shared.MGDispatchers
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flowOn
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.flow.mapLatest
import java.time.LocalDate
import javax.inject.Inject

class CareplanActivitiesRepository @Inject constructor(
    pregnancyPreferences: PregnancyPreferencesDataSource,
    private val appointmentsDataSource: AppointmentsOfflineDataSource,
    @Dispatcher(MGDispatchers.IO) private val ioDispatcher: CoroutineDispatcher,
) {
    private val dueDateMethod = pregnancyPreferences.data.map { data ->
        val edd = data.expectedDueDate
        val lmp = data.lastMenstruationPeriod

        when {
            edd != null -> DueDateMethod.ExpectedDueDate(edd)
            lmp != null -> DueDateMethod.LastMenstruationPeriod(lmp)
            else -> null
        }
    }

    fun getActivities(): Flow<SplitCareplanActivities> = dueDateMethod.mapLatest {
        val lmp = it?.lastMenstruationPeriod ?: LocalDate.now()
        val now = LocalDate.now()

        val careplanActivities = appointmentsDataSource.getCareplanTemplate().toDomainModel()

        val (past, future) = careplanActivities.activities.partition { activity -> lmp.plusWeeks(activity.untilWeek.toLong()).isBefore(now) }

        SplitCareplanActivities(past = past, future = future)
    }.flowOn(ioDispatcher)
}
